/**
 *  Button.h
 *  
 *  A Button object is in most aspects the same as a hotspot, but it provides support for a pressed and released state. 
 *  This state property helps make nice looking graphical buttons with feedback. 
 *
 *  When inheriting from this class to create your buttons, simple override/provide implementation for any or all desired
 *  handleTouch methods of three, and at least paintReleased. If you want you button to have graphical feedback when you touch
 *  it, implement paintPressed. If you want your button to have graphical feedback when it is in focus, implement 
 *  paintReleasedFocused(). If you want your button to have graphical feedback different from when normally pressed when it
 *  gets touched while in focus, implement paintPressedFocused().
 *
 *  Created by Jaeder Patater on 11/6/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef BUTTON_H
#define BUTTON_H

#include "HotSpot.h"

class Button : public HotSpot {
protected:
    bool pressed; //indicates whether or not the button is pressed
    char * name; //The name of the button. Used with MenuScreen to get pointers to buttons via the findButton method. Could be used to display a title on the button, depending on how you implement paint.
    
    void init();
    void init(char * name);
    void init(const Button & other);
    
    //constructors are now protected, because with this new, pure model (7 November 2005 and beyond), you
    //are expected to create a class for each button-handler combo (for each handler, you create a button class)
    // i.e. NewGameButton, KickButton, SaveButton, etc.
    
    /**
     *  Button(char * name, uint16 x, uint16 y, uint16 width, uint16 height)
     *
     *  Creates a Button based on the specified name and bbox
     *
     */
    Button(char * name, uint16 x, uint16 y, uint16 width, uint16 height);
    
    Button();                               //default constructor
    Button(const Button & other);           //copy constructor
    
public:
    
    virtual ~Button();                      //deconstructor
    Button operator=(const Button & other); //= operator
    
    /**
     *  press()
     *
     *  Changes the button's state to pressed.
     *
     */
    virtual void press();
    
    /**
     *  release()
     *
     *  Changes the button's state to released and executes the button code (handleTouch())
     *
     */
    virtual void release();
    
    /**
     *  getName()
     *
     *  Returns the name of the button
     *
     */
    char * getName();
    
    /**
     *  paint()
     *
     *  Paints the object depending on the state by calling code in paintPressed() or paintReleased()
     *
     */
    virtual void paint();
    
    /**
     *  recieveEvent(UIEvent * event)
     *
     *  Handles the reception of events passed to this Button. It uses the state of a TouchEvent object to provide press and release functionality.
     *
     */
    virtual void recieveEvent(UIEvent * event);
    
    /**
     *  handleTouchUp()
     *
     *  Put the code you want to execute on button release into this method
     *
     */
    virtual void handleTouchUp();
    
    /**
     *  handleTouchDown()
     *
     *  Put the code you want to execute on button press into this method
     *
     */
    virtual void handleTouchDown();
    
    /**
     *  handleTouchHeld()
     *
     *  Put the code you want to execute on button hold into this method
     *
     */
    virtual void handleTouchHeld();
    
    /**
     *  paintPressed()
     *
     *  Paints in the pressed state.
     *  
     *  Put your paint code here.
     *
     */
    virtual void paintPressed();
    
    /**
     *  paintReleased()
     *
     *  Paints the object in the released state.
     *
     *  Put your paint code here.
     *
     */
    virtual void paintReleased();
    
    /**
     *  paintPressedFocused()
     *
     *  Paints the object in the focused and pressed state.
     *
     *  Put your paint code here (only necessary if you want the object to show focus). Default (paintPressed())
     *
     */
    virtual void paintPressedFocused();
    
    /**
     *  paintReleasedFocused()
     *
     *  Paints the object in the focused and released state.
     *
     *  Put your paint code here (only necessary if you want the object to show focus).
     *
     */
    virtual void paintReleasedFocused();
    
};

#endif

