/**
 *  Screen.h
 *  
 *  A class representing a Nintendo DS screen.
 *
 *  In application, a Screen object is used as the main container for all elements, the top level component per se.
 *
 *  Default graphical components will use methods in this class to draw. Feel free to change the implementation of
 *  the Screen class to suit your graphical needs, or change the default components to draw how you'd like in your graphical environment.
 *
 *
 *  WARNING: This class needs a MAJOR overhaul. It was written by a dude (Patater) who had no clue about the 2D gfx engine on the DS. He just fiddled until things worked and asked for tips on #dsdev.
 *  Most things in 16bit gfx mode don't work properly. I'd recommend using your own gfx routines or revising this class if you plan on using it. Discuss on #dsdev with Patater or email guikit@patatersoft.info .
 *
 *
 *  Created by Jaeder Patater on 8/5/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef SCREEN_H
#define SCREEN_H

#include <stdio.h>

#include "nds.h"
#include "nds/arm9/console.h"

#include "UIElement.h"
#include "UIElementList.h"
#include "ScreenType.h"
#include "ScreenMode.h"
#include "HotSpot.h"
#include "KeyInput.h"

class Screen : public UIElement {
protected:
    
    /**
     *  screenType
     *
     *  used to store the type of screen. main or sub.
     *
     */
    ScreenType screenType;
    
    /**
     *  ScreenMode screenMode
     *
     *  Allows the screen to draw differently depending on the mode its in (framebuffer, mode 5 exrot bg in 16bit, mode 5 exrot bg in 256 color paletted, etc)
     *
     */
    ScreenMode screenMode;
    
    /**
     *  consoleColor
     *
     *  Used when initializing the display in CONSOLE mode. It is the color of the text on console.
     *
     */
    uint16 consoleColor;
    
    /**
     *  lastTouched
     *
     *  Used in the recieveEvent method when the touch has left the bounds of a HotSpot object. This is the way that touch events of status exit are generated. (Enables certain button handling behavior.)
     *
     */
    HotSpot * lastTouched;
    
    /**
     *  lastTouchEventStatus
     *
     *  Keeps track of the last event's status, given it was a touch event, for use in generating touch events of status exit.
     *
     */
    //UIEventStatus lastTouchEventStatus;
    
    //bool enabled; //used to disable or enable a screen
    
public:
    
    /**
     *  Screen(ScreenType _screenType)
     *
     *  Creates a Screen object given a screen type and mode.
     *
     */
    Screen(ScreenType _screenType, ScreenMode _screenMode);
    
    /**
     *  Screen()
     *
     *  Default constructor. Default ScreenType is MAIN and default ScreenMode is FRAME_BUFFER.
     *
     */
    Screen();
    
    Screen(const Screen & other);           //copy constructor
    virtual ~Screen();                      //deconstructor
    Screen operator=(const Screen & other); //= operator
    
    /**
     *  initializeDisplay()
     *
     *  Sets up the screen to display in the current mode.
     *
     */
    void initializeDisplay();
    
    /**
     *  initializeDisplay(ScreenMode _screenMode)
     *
     *  Sets up the screen to display in the specified mode.a palletized bitmap background with size 256x256.
     *
     */
    void initializeDisplay(ScreenMode _screenMode);
    
    /**
     *  initializeConsolePrint(ScreenType screenType, uint16 color)
     *
     *  Initializes the screen for console print with a given color.
     *
     */
    void initializeConsolePrint(uint16 color);
    
    /**
     *  setScreenType(ScreenType _screenType)
     *
     *  Sets the screen to the specified type (moving it to the main or sub screen).
     *
     */
    void setScreenType(ScreenType _screenType);
    
    /**
     *  setScreenMode(ScreenType _screenType)
     *
     *  Sets the screen to the specified type (moving it to the main or sub screen).
     *
     */
    void setScreenMode(ScreenMode _screenMode);
    
    /**
     *  loadPalette(const uint16 * palette)
     *
     *  Loads the palette into BG_PALETTE or BG_PALETTE_SUB depending on the screen type. Does nothing if mode is not paletted.
     *
     */
    void loadPalette(const uint16 * palette);
    
    /**
     *  drawPixelBackground(uint8 paletteReference, int x, int y)
     *
     *  Draw a pixel to the background with a given paletteReference (for example: 0xFF for last color in palette). Does nothing if mode is not paletted.
     *
     */
    void drawPixelBackground(uint8 paletteReference, int x, int y);
    
    /**
     *  drawPixelBackground(uint16 color, int x, int y)
     *
     *  Draw a pixel to the background with a given color (for example: 0xD836 for PataterSoft purple). Does nothing if mode is paletted.
     *
     */
    void drawPixelBackground(uint16 color, int x, int y);
    
    /**
     *  drawPixelForeground(uint8 paletteReference, int x, int y)
     *
     *  Draw a pixel to the background with a given paletteReference (for example: 0xFF for last color in palette). Does nothing if mode is not paletted.
     *
     */
    void drawPixelForeground(uint8 paletteReference, int x, int y);
    
    /**
     *  drawPixelForeground(uint16 color, int x, int y)
     *
     *  Draw a pixel to the background with a given color (for example: 0xD836 for PataterSoft purple). Does nothing if mode is paletted.
     *
     */
    void drawPixelForeground(uint16 color, int x, int y);
    
    /**
     *  fillBackground(uint8 paletteReference)
     *
     *  Fills the background with a given paletteReference (for example: 0xFF for last color in palette). Does nothing if mode is not paletted.
     *
     */
    void fillBackground(uint8 paletteReference, uint32 size);
    
    /**
     *  fillForeground(uint8 paletteReference)
     *
     *  Fills the background with a given paletteReference (for example: 0x00 for transparent). Does nothing if mode is not paletted.
     *
     */
    void fillForeground(uint8 paletteReference, uint32 size);
    
    /**
     *  fillBackground(uint16 color)
     *
     *  Fills the background with a given paletteReference (for example: 0xD836 for PataterSoft purple). Does nothing if mode is paletted.
     *
     */
    void fillBackground(uint16 color, uint32 size);
    
    /**
     *  fillForeground(uint16 color)
     *
     *  Fills the background with a given paletteReference (for example: 0x8000 for black). Does nothing if mode is paletted.
     *
     */
    void fillForeground(uint16 color, uint32 size);
    
    /**
     *  drawBackground(const uint16 data[], int x, int y, int width, int height)
     *
     *  Draws an image on the screen into the specified destination on the background.
     *
     */
    void drawBackground(const uint16 data[], int x, int y, int width, int height);
    
    /**
     *  drawForeground(const uint16 data[], int x, int y, int width, int height)
     *
     *  Draws an image on the screen into the specified destination on the foreground.
     *
     */
    void drawForeground(const uint16 data[], int x, int y, int width, int height);
    
    /**
     *  drawBackground(const uint16 data[], int sourceX, int sourceY, int sourceWidth, int sourceHeight, int destX, int destY, int destWidth, int destHeight)
     *
     *  Draws an image from the specified source pixels into the specified destination on the background.
     *
     *  !!!Warning: does not yet support a sourceWidth and sourceHeight different from destWidth and destHeight
     *
     */
    void drawBackground(const uint16 data[], int sourceX, int sourceY, int sourceWidth, int sourceHeight, int destX, int destY, int destWidth, int destHeight);
    
    /**
     *  drawForeground(const uint16 data[], int sourceX, int sourceY, int sourceWidth, int sourceHeight, int destX, int destY, int destWidth, int destHeight)
     *
     *  Draws an image from the specified source pixels into the specified destination on the foreground.
     *
     *  !!!Warning: does not yet support a sourceWidth and sourceHeight different from destWidth and destHeight
     *
     */
    void drawForeground(const uint16 data[], int sourceX, int sourceY, int sourceWidth, int sourceHeight, int destX, int destY, int destWidth, int destHeight);
    
    /**
     *  fillRectForeground(uint16 color, int x, int y, int width, int height)
     *
     *  Fills a rectangle of specified color onto the foreground
     *
     */
    void fillRectForeground(uint16 color, int x, int y, int width, int height);
    
    /**
     *  fillRectBackground(uint16 color, int x, int y, int width, int height)
     *
     *  Fills a rectangle of specified color onto the foreground
     *
     */
    void fillRectBackground(uint16 color, int x, int y, int width, int height);
    
    /**
     *  drawRectForeground(uint16 color, int x, int y, int width, int height)
     *
     *  Draws a rectangle of specified color onto the foreground
     *
     */
    void drawRectForeground(uint16 color, int lineWidth, int x, int y, int width, int height);
    
    /**
     *  drawRectBackground(uint16 color, int x, int y, int width, int height)
     *
     *  Draws a rectangle of specified color onto the background
     *
     */
    void drawRectBackground(uint16 color, int lineWidth, int x, int y, int width, int height);
    
    /**
     *  drawRoundedRectForeground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius)
     *
     *  Draws a rounded rectangle of specified color, linewidth, and x,y radius, onto the foreground with upper left rect coordinate at specified x,y
     *
     */
    void drawRoundedRectForeground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius);
    
    /**
     *  drawRoundedRectBackground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius)
     *
     *  Draws a rounded rectangle of specified color, linewidth, and x,y radius, onto the background with upper left rect coordinate at specified x,y
     *
     */
    void drawRoundedRectBackground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius);
    
    /**
     *  drawOvalForeground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius)
     *
     *  Draws an oval of specified x,y radius onto the foreground with upper left rect coordinate at specified x,y
     *
     */
    void drawOvalForeground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    /**
     *  drawOvalBackground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius)
     *
     *  Draws an oval of specified x,y radius onto the background with upper left rect coordinate at specified x,y
     *
     */
    void drawOvalBackground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    /**
     *  drawCircleForeground(uint16 color, int lineWidth, int x, int y, int radius)
     *
     *  Draws an circle of specified radius onto the foreground with upper left rect coordinate at specified x,y
     *
     */
    void drawCircleForeground(uint16 color, int lineWidth, int x, int y, int radius);
    
    /**
     *  drawCircleBackground(uint16 color, int lineWidth, int x, int y, int radius)
     *
     *  Draws an circle of specified radius onto the background with upper left rect coordinate at specified x,y
     *
     */
    void drawCircleBackground(uint16 color, int lineWidth, int x, int y, int radius);
    
    /**
     *  fillRoundedRectForeground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius)
     *
     *  Fills a rounded rectangle of specified color, linewidth, and x,y radius, onto the foreground
     *
     */
    void fillRoundedRectForeground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius);
    
    /**
     *  fillRoundedRectBackground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius)
     *
     *  Fills a rounded rectangle of specified color, linewidth, and x,y radius, onto the background
     *
     */
    void fillRoundedRectBackground(uint16 color, int lineWidth, int x, int y, int width, int height, int xRadius, int yRadius);
    
    /**
     *  fillOvalForeground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius)
     *
     *  Fills an oval of specified x,y radius onto the foreground with upper left rect coordinate at specified x,y
     *
     */
    void fillOvalForeground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    /**
     *  fillOvalBackground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius)
     *
     *  Fills an oval of specified x,y radius onto the background with upper left rect coordinate at specified x,y
     *
     */
    void fillOvalBackground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    /**
     *  fillCircleForeground(uint16 color, int lineWidth, int x, int y, int radius)
     *
     *  Fills an circle of specified radius onto the foreground with upper left rect coordinate at specified x,y
     *
     */
    void fillCircleForeground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    /**
     *  fillCircleBackground(uint16 color, int lineWidth, int x, int y, int radius)
     *
     *  Fills an circle of specified radius onto the background with upper left rect coordinate at specified x,y
     *
     */
    void fillCircleBackground(uint16 color, int lineWidth, int x, int y, int xRadius, int yRadius);
    
    virtual void recieveEvent(UIEvent * event);
    
    /**
     *  paint()
     *  
     *  Default paint behavior of a screen is to simply paint its contained elements
     *
     */
    virtual void paint();
    
    /**
     *  handleIdle()
     *
     *  Whenever the DS is idling (no events), this function is called
     *
     */
    virtual void handleIdle();
    
};

#endif

