/**
 *  UIEventManager.h
 *  
 *  This class manages all elements contained within it, passing events to them as it recieves them and as it sees fit.
 *
 *  One application of a class of this type would be to recieve touch events and pass them onto those elements within
 *  the specified coordinates. A screen class could be the highest level container in this regard, passing onto buttons
 *  placed onto the screen or windows within the screen to send to buttons or menus contained within them.
 *
 *  Created by Jaeder Patater on 8/5/05.
 *  Copyright 2005 PataterSoft. All rights reserved.
 *
 */

#ifndef UI_EVENT_MANAGER_H
#define UI_EVENT_MANAGER_H

#include "nds.h"

#include "UIEventReciever.h"
#include "LinkedList.h"

//not included for help resolving interdependency issues
//#include "UIElementList.h"
//#include "UIElement.h"
//#include "KeyInput.h"

//predeclarations
class UIElement;
class UIElementList;
class KeyInput;

class UIEventManager : public UIEventReciever {
protected:
    
    UIElementList * elementList; //the list of all contained UIElement objects
    
    KeyInput * keyInput; //pointer should hold the current key input object (maybe we shouldn't allow keys to be in the element list, or perhaps it shouldn't be a UIElement)
    
    UIElement * focusedElement; //current UIElement object in focus
    int focus; //the index in the elementList of the focusedElement
    
public:
    
    UIEventManager(UIElementList * _elementList);
    
    UIEventManager();                                       //default constructor
    UIEventManager(const UIEventManager & other);           //copy constructor
    virtual ~UIEventManager();                              //destructor
    UIEventManager operator=(const UIEventManager & other); //= operator
    
    /**
     *  addElement(UIElement * element)
     *
     *  Adds a UIElement to the manager
     *
     */
    virtual void addElement(UIElement * element);
    
    /**
     *  insertElementBefore(UIElement * index, UIElement * element)
     *
     *  Inserts the specified element before the index element. If the index is not contained by the manager, the element is not inserted.
     *
     */
    virtual void insertElementBefore(UIElement * index, UIElement * element);
    
    /**
     *  insertElementAfter(UIElement * index, UIElement * element)
     *
     *  Inserts the specified element after the index element. If the index is not contained by the manager, the element is not inserted.
     *
     */
    virtual void insertElementAfter(UIElement * index, UIElement * element);
    
    /**
     *  removeElement(UIElement * element)
     *
     *  Removes an element from the manager
     *
     */
    virtual void removeElement(UIElement * element);
    
    /**
     *  setKeys(KeyInput * _keyInput)
     *
     *  Sets the specified _keyInput to be the current keyInput
     *
     */
    virtual void setKeys(KeyInput * _keyInput);
    
    /**
     *  get(int i)
     *
     *  Returns index i of the list, NULL if invalid index
     *
     */
    UIElement * get(int i) const;
    
    /**
     *  getNumContainedElements()
     *
     *  Returns the number of contained elements.
     *
     */
    int getNumContainedElements();
    
    /**
     *  setElementList(UIElementList * elementList)
     *
     *  Sets the UIElementList of this to the specified UIElementList
     *
     */
    virtual void setElementList(UIElementList * _elementList);
    
    /**
     *  clearElementList()
     *
     *  Clears the list of UIElement objects
     *
     */
    virtual void clearElementList();
    
    /**
     *  setFocus(UIElement * element)
     *
     *  Sets the focus to the specified element, ignoring the command if the element is not contained in this UIEventManager.
     *
     */
    virtual void setFocus(UIElement * element);
    
    /**
     *  setFocusUp()
     *
     *  Cycles through focus in the up direction.
     *  
     *  up and down directions simply numbers in element list, could be overridden to support left and right with HotSpots (TouchRect) based on their locations
     *  the left and right behaviors could be simulated by a cleverly layed out element list as well
     *
     */
    virtual void setFocusUp(); //"look up!"
    
    /**
     *  setFocusDown()
     *
     *  Cycles through focus in the down direction.
     *  
     *  (up and down directions simply numbers in element list, could be overridden to support left and right with HotSpots (TouchRect) based on their locations)
     *  the left and right behaviors could be simulated by a cleverly layed out element list as well
     *
     */
    virtual void setFocusDown();
    
    /**
     *  touchFocus()
     *
     *  Generates and sends a touch event to the UIElement object currently in focus.
     *
     */
    virtual void touchFocus();
    
    /**
     *  sendEventFocus(UIEvent * event)
     *
     *  Sends the specified event to the UIElement object currently in focus
     *
     */
    virtual void sendEventFocus(UIEvent * event);
    
    /**
     *  getFocusElement()
     *
     *  Returns the element currently in focus
     *
     */
    virtual UIElement * getFocusElement();
    
    /**
     *  clearFocus()
     *
     *  Clears the focus of this manager.
     *
     *  focusedElement is set to NULL and focus index remains the same (same so that if you resume focus, you start where you left off)
     *
     */
    virtual void clearFocus(); //"Don't look anywhere!" "Eh?" "Shut your eyes!"
    
    /**
     *  resumeFocus()
     *
     *  Resumes focus on whatever index you were last focusing on (whether or not this is the same object is not guarunteed)
     *
     */
    virtual void resumeFocus();
    
    /**
     *  setFocusFirst()
     *
     *  Sets the focus to the first focusable object in the elementList
     *
     */
    virtual void setFocusFirst();
    
    /**
     *  setFocusLast
     *
     *  Sets the focus to the last focusable object in the elementList
     *
     */
    virtual void setFocusLast();
    
    /**
     *  paintContainedElements()
     *
     *  Calls paint() on all the elements contained within this screen.
     *
     */
    virtual void paintContainedElements();
    
    /**
     *  recieveEvent(UIEvent * event);
     *
     *  Recieves a UIEvent and calls the appropriate handler.
     *
     */
    virtual void recieveEvent(UIEvent * event);
    
};

#endif

