/**
 *  UIEventManager.cpp
 *  
 *
 *  Created by Jaeder Patater on 8/5/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#include "UIEventManager.h"

//included for help resolving interdependency issues
#include "UIElementList.h"
#include "UIElement.h"
#include "KeyInput.h"

UIEventManager::UIEventManager(UIElementList * _elementList) {
    setElementList(_elementList);
    keyInput = NULL;
}

UIEventManager::UIEventManager() {
    elementList = new UIElementList();
    keyInput =  NULL;
}

UIEventManager::UIEventManager(const UIEventManager & other) {
    elementList = new UIElementList(*(other.elementList)); //bad, slow, should just point to it, but won't be able to until we start using smart pointers
    keyInput =  NULL; //bad, should copy from other.elementList, but won't be able to until we start using smart pointers or whatnot
}

UIEventManager::~UIEventManager() {
    if (elementList != NULL) {
        delete elementList;
        elementList = NULL;
    }
}

UIEventManager UIEventManager::operator=(const UIEventManager & other) {
    elementList = new UIElementList(*(other.elementList));
    
    return *this;
}

void UIEventManager::addElement(UIElement * element) {
    if (elementList != NULL) {
        if (element != NULL) { //don't add a NULL element to the list
            elementList->add(element);
            
            //set parent container
            element->setParentContainer(this); //undefined behavior (on NDS, any other system will throw NULL pointer exception) here
        }
    }
}

void UIEventManager::insertElementBefore(UIElement * index, UIElement * element) {
    if ((elementList != NULL) && (element != NULL)) {
        elementList->insertBefore(index, element);
    }
}

void UIEventManager::insertElementAfter(UIElement * index, UIElement * element) {
    if ((elementList != NULL) && (element != NULL)) {
        elementList->insertAfter(index, element);
    }
}

void UIEventManager::removeElement(UIElement * element) {
    if (elementList != NULL) {
        if (element != NULL) {
            elementList->remove(element);
            //parent container issues resolved by the element list
        }
    }
}

void UIEventManager::setKeys(KeyInput * _keyInput) {
    keyInput = _keyInput;
    keyInput->setParentContainer(this); //even though not really in the element list, the manager should be this parent (for setting focus on things, controlling elements within the list, etc.)
}

UIElement * UIEventManager::get(int i) const {
    if (elementList != NULL) {
        return elementList->get(i);
    } else {
        return NULL;
    }
}

int UIEventManager::getNumContainedElements() {
    return elementList->getSize();
}

void UIEventManager::setElementList(UIElementList * _elementList) {
    elementList = new UIElementList(*_elementList);
}

void UIEventManager::clearElementList() {
    if (elementList != NULL) {
        elementList->clear();
        //parent container issues resolved by the element list
    }
}

void UIEventManager::setFocus(UIElement * element) {
    int elementNum = elementList->find(element);
    if ((elementNum != -1) && (element->canRecieveFocus())) { //if the element was found and it can recieve focus
        UIElement * temp = focusedElement;
        focusedElement = element;
        //when focus is removed from an element, unpress it by sending it a touch exit event (which also calls paint() and forces the element to check if its in focus)
        temp->recieveEvent(new TouchEvent(STATUS_EXIT, 0, 0));
        focus = elementNum;
        focusedElement->paint();
    }
}

void UIEventManager::setFocusUp() {
    //initial focus state check
    if ((focusedElement == NULL) && (focus == 0)) { //if initial focus state
        //focus on first available object
        focusedElement = elementList->get(focus);
        if ((focusedElement != NULL) && (focusedElement->canRecieveFocus())) {
            focusedElement->paint();
            return;
        }
    }
    
    //until you get an element that can recieve focus, cycle up through the list
    
    UIElement * element = focusedElement;
    int startFocus = focus; //we have complete a loop when focus is one below original value
    
    //remove focus on whatever is currently in focus and paint it
    focusedElement = NULL;
    if (element != NULL) {
        //when focus is removed from an element, unpress it by sending it a touch exit event (which also calls paint() and forces the element to check if its in focus)
        element->recieveEvent(new TouchEvent(STATUS_EXIT, 0, 0));
    }
    
    do {
        ++focus;
        if (focus >= elementList->getSize()) { //we increased out of the lists bounds
            focus = 0; //restart at index zero
        }
        
        element = elementList->get(focus);
    } while ((element->canRecieveFocus() != true) && (focus != startFocus));
    
    //if startFocus is the same as focus, the effect will be the same as setting focusedElement to elementList->get(focus)
    focusedElement = element;
    
    //paint the element when its focus is set
    focusedElement->paint();
}

void UIEventManager::setFocusDown() {
    //initial focus state check
    if ((focusedElement == NULL) && (focus == 0)) { //if initial focus state
        //focus on first available object
        focusedElement = elementList->get(focus);
        if ((focusedElement != NULL) && (focusedElement->canRecieveFocus())) {
            //paint the element when its focus is set
            focusedElement->paint();
            return;
        }
    }
    
    //until you get an element that can recieve focus or have complete the list, cycle down through the list
    
    UIElement * element = focusedElement;
    int startFocus = focus; //we have complete a loop when focus is one below original value
    
    //remove focus on whatever is currently in focus and paint it
    focusedElement = NULL;
    if (element != NULL) {
        //when focus is removed from an element, unpress it by sending it a touch exit event (which also calls paint() and forces the element to check if its in focus)
        element->recieveEvent(new TouchEvent(STATUS_EXIT, 0, 0));
    }
    
    do {
        --focus;
        if (focus < 0) { //we decreased below the list
            //start at the back of the list
            focus = elementList->getSize() - 1;
        }
        
        element = elementList->get(focus);
        
    } while((element->canRecieveFocus() != true) && (focus != startFocus));
    
    //if startFocus is the same as focus, the effect will be the same as setting focusedElement to elementList->get(focus)
    focusedElement = element;
    
    //paint the element when its focus is set
    focusedElement->paint();
}

void UIEventManager::touchFocus() {
    //if there is a way to get the position of the object in reference to its parent container, you should to that and then send the touch event with those coordinates
    //i.e. A button is in a screen at 24x60. touchFocus() should look into the buttons parent container (itself), find the element's position, and pass that as the coords (instead of 0,0)
    sendEventFocus(new TouchEvent(STATUS_DOWN, 0, 0));
    swiWaitForVBlank();
    sendEventFocus(new TouchEvent(STATUS_UP, 0, 0));
}

void UIEventManager::sendEventFocus(UIEvent * event) {
    if (focusedElement != NULL) {
        focusedElement->recieveEvent(event);
    }
}

UIElement * UIEventManager::getFocusElement() {
    return focusedElement;
}

void UIEventManager::clearFocus() {

    UIElement * element = focusedElement;
    focusedElement = NULL;
    if (element != NULL) {
        //when focus is removed from an element, unpress it by sending it a touch exit event (which also calls paint() and forces the element to check if its in focus)
        element->recieveEvent(new TouchEvent(STATUS_EXIT, 0, 0));
    }
}

void UIEventManager::resumeFocus() {
    focusedElement = elementList->get(focus);
    
    if (focusedElement == NULL) { //if NULL
        setFocusUp();
    }
    
    if (focusedElement != NULL) {
        //if not NULL, make sure it can recieve focus
        if (focusedElement->canRecieveFocus() != true) { //if it can't be focused, cycle through until you reach an element that can recieve focus
            
            setFocusUp();
            
            if (focusedElement->canRecieveFocus() != true) {
                //there are no focusable elements
                focusedElement = NULL;
            }
        }
    }
    
    //focusedElement will be NULL here if there are no focusable elements
    if (focusedElement != NULL) {
        focusedElement->paint();
    }
}

void UIEventManager::setFocusFirst() {
    focus = 0;
    
    //check if this one is good
    focusedElement = elementList->get(focus);
    if (focusedElement != NULL) {
        if (focusedElement->canRecieveFocus()) {
            return;
        }
    }
    
    
    setFocusUp();
}

void UIEventManager::setFocusLast() {
    focus = elementList->getSize() - 1;
    
    //check if this one is good
    focusedElement = elementList->get(focus);
    if (focusedElement != NULL) {
        if (focusedElement->canRecieveFocus()) {
            return;
        }
    }
    
    setFocusDown();
}

void UIEventManager::paintContainedElements() {
    //iterate through each element in the list and call paint on it, starting from the lower most element (lower most element is lowest index, 0)
    //The reason for this paint order is so that lower elements are painted under, and above elements are painted above.
    /*if (elementList != NULL) {
        if (elementList->IsEmpty() == false) { //make sure the list isn't empty before trying to operate on it
            UIElement * element = NULL;
            for (int i = 0; i < elementList->getSize(); i++) {
                element = elementList->get(i);
                if (element != NULL) {
                    elementList->get(i)->paint();
                }
            }
        }
    }*/
    if (elementList != NULL) {
        const LinkedList<UIElement *> * list= elementList->getList();
        if (!(list->IsEmpty())) { //make sure the list isn't empty before trying to operate on it
            LLNode<UIElement *> * node = list->GetFirst();
            while (node != NULL) {
                node->GetValue()->paint();
                node = node->GetNext();
            }
        }
    }
}

void UIEventManager::recieveEvent(UIEvent * event) {
    //nothing
}

